@@ -5,7 +5,7 @@ use warnings;
 
 use File::Spec ();
 our @ISA = qw(File::Spec);
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 sub linked { 
     my($spec, $link) = @_;
@@ -21,11 +21,25 @@ sub relative_to_file {
     return $spec->catdir($dir,$path);
 }
 
+sub splitlast {
+    my($spec, $dir) = @_;
+    my @path = $spec->splitdir($dir);
+    my $found = scalar @path;
+    while( --$found ) { last if length $path[$found] } 
+    return ($spec->catdir(@path[0..$found-1]), $path[$found]);
+}
+
 sub chopfile {
     my($spec, $file) = @_;
-    my($vol, $dir) = $spec->splitpath($file);	# discard file component
-    return $spec->catpath($vol, $dir, '') if $vol;	# UNIX short cut
-    return $dir if $dir;
+    my($vol, $dir);
+    ($vol, $dir, $file) = $spec->splitpath($file);	
+						# discard file component
+    unless( length $file ) {
+	($dir, $file) = $spec->splitlast($dir);
+	die unless length $file;
+    }
+    return $spec->catpath($vol, $dir, '')	if length $vol;
+    return $dir					if length $dir;
     return $spec->curdir;
 }
 
@@ -41,11 +55,52 @@ sub resolve {
 
 sub full_resolve {
     my($spec, $file) = @_;
-    my $abs = do { require Cwd; Cwd::abs_path($file) };
+    my $path = $spec->resolve_path($file);
+    return defined $path ? $path : $spec->resolve_all($file);
+}
+
+sub resolve_path {
+    my($spec, $file) = @_;
+    my $path = eval { require Cwd; Cwd::abs_path($file) };
+    return unless defined $path; 
     return $spec->file_name_is_absolute($file)
-	    ? $abs : $spec->abs2rel($abs);
+	    ? $path : $spec->abs2rel($path);
 } 
     
+sub resolve_all {
+    my($spec, $file) = @_;
+    my %seen;
+    while( -l $file ) {
+	return if $seen{$spec->canonpath($file)}++;
+	return unless defined($file = $spec->linked($file));
+    }
+    my($vol, $dir, @temp);
+    ($vol, $dir, $file) = $spec->splitpath($file);
+    while( length $dir ) {
+	my $path = $spec->catpath($vol, $dir, '');
+	if( -l $path ) {
+	    return if $seen{$spec->canonpath($path)}++;
+	    return unless defined($path = $spec->linked($path));
+            ($vol, $dir) = $spec->splitpath($path, 1);
+	}
+	else {
+            ($dir, my $last) = $spec->splitlast($dir);	
+	    unshift @temp, $last;
+	}
+    }
+    my @path;
+    for my $path (@temp) {
+ 	if( $path eq $spec->curdir ) { next if @path } 
+	elsif( $path eq $spec->updir ) {
+	    if( @path and $path[-1] ne $spec->updir ) {
+		pop @path; next 
+	    }
+	}
+	push @path, $path
+    }	
+    $spec->catpath($vol, $spec->catdir(@path), $file);
+}
+
 1;
 __END__
 # Below is stub documentation for your module. You'd better edit it!
@@ -63,6 +118,8 @@ File::Spec::Link - Perl extension for reading and resolving symbolic links
     my $newname = File::Spec::Link->relative_to_file($path, $link);
   
     my $realname = File::Spec::Link->full_resolve($file);
+    my $realname = File::Spec::Link->resolve_path($file);
+    my $realname = File::Spec::Link->resolve_all($file);
 
 =head1 DESCRIPTION
 
@@ -91,10 +148,21 @@ and returning (the volumne and) directory parts.
 Returns the path of C<$path> relative to the directory of file
 C<$file>.  If C<$path> is absolute, just returns C<$path>.
 
+=item C<< ->resolve_all($file) >>
+
+Returns the filename of C<$file> with all links in the path resolved,
+wihout using C<Cwd>.
+
 =item C<< ->full_resolve($file) >>
 
 Returns the filename of C<$file> with all links in the path resolved.
 
+This sub tries to use C<Cwd::abs_path> via C<< ->resolve_path >>.
+
+=item C<< ->resolve_path($file) >>
+
+Returns the filename of C<$file> with all links in the path resolved.
+
 This sub uses C<Cwd::abs_path> and is independent of the rest of
 C<File::Spec::Link>. 
 
